@@ -48,9 +48,11 @@
 
 // Jovie includes.
 #include "talkermgr.h"
+#include "talkercode.h"
 // define spd_debug here to avoid a link error in speech-dispatcher 0.6.7's header file for now
 #define spd_debug spd_debug2
 #include "speaker.h"
+#include "jovietrayicon.h"
 
 #include "kspeechadaptor.h"
 
@@ -60,10 +62,12 @@ class JoviePrivate
 {
     JoviePrivate()
     {
+        trayIcon = new JovieTrayIcon();
     }
 
     ~JoviePrivate()
     {
+        delete trayIcon;
     }
 
     friend class Jovie;
@@ -73,6 +77,12 @@ protected:
     * The DBUS sender ID of last application to call KTTSD.
     */
     QString callingAppId;
+    
+    /*
+    * The tray icon.
+    */
+    JovieTrayIcon *trayIcon;
+  
 };
 
 /* Jovie Class ========================================================= */
@@ -137,6 +147,16 @@ void Jovie::setDefaultTalker(const QString &defaultTalker)
     Speaker::Instance()->getAppData(callingAppId())->setDefaultTalker(defaultTalker);
 }
 
+void Jovie::setCurrentTalker(const TalkerCode &talker)
+{
+    Speaker::Instance()->setOutputModule(talker.outputModule());
+    Speaker::Instance()->setLanguage(TalkerCode::languageCodeToLanguage(talker.language()));
+    Speaker::Instance()->setVoiceType(talker.voiceType());
+    Speaker::Instance()->setVolume(talker.volume());
+    Speaker::Instance()->setSpeed(talker.rate());
+    Speaker::Instance()->setPitch(talker.pitch());
+}
+
 int Jovie::defaultPriority()
 {
     return Speaker::Instance()->getAppData(callingAppId())->defaultPriority();
@@ -264,6 +284,11 @@ QStringList Jovie::languagesByModule(const QString & module)
     return Speaker::Instance()->languagesByModule(module);
 }
 
+QStringList Jovie::getPossibleTalkers()
+{
+    return Speaker::Instance()->getPossibleTalkers();
+}
+
 void Jovie::setSpeed(int speed)
 {
     if (speed < -100 || speed > 100) {
@@ -482,6 +507,8 @@ void Jovie::reinit()
     if (ready()) {
         QDBusConnection::sessionBus().registerObject(QLatin1String( "/KSpeech" ), this, QDBusConnection::ExportAdaptors);
     }
+    
+    d->trayIcon->slotUpdateTalkersMenu();
 }
 
 void Jovie::setCallingAppId(const QString& appId)
